﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ozeki.Media;
using Ozeki.Media.MediaHandlers;
using Ozeki.VoIP;
using System.Windows.Forms;

namespace Voice_IVR.Model
{
    sealed class CallSoundEngine : IDisposable
    {
        private static Speaker Speaker;
        private static Microphone Microphone;
        private static AudioMixerMediaHandler SpeakerMixer;
        private static AudioMixerMediaHandler MicropohoneMixer;
        private static MediaConnector FocusConnector;

        public static void InitMicropohoneAndSpeaker()
        {
            Speaker =  Speaker.GetDefaultDevice();
            if (Speaker != null)
                Speaker.Start();

            Microphone =  Microphone.GetDefaultDevice();
            if (Microphone != null)
                Microphone.Start();

            //some devices are missing
            string message = String.Empty;

            if (Microphone == null)
                message += "You have no microphone attached to your computer, please note that your partner will not hear your voice.\n";
            if (Speaker == null)
                message += "You have no speaker attached to your computer, please note that you will not hear your partner.\n";

            if (message != String.Empty)
                MessageBox.Show(message, "Some devices are missing", MessageBoxButtons.OK, MessageBoxIcon.Information);

            FocusConnector = new MediaConnector();

            SpeakerMixer = new AudioMixerMediaHandler();
            MicropohoneMixer = new AudioMixerMediaHandler();

            FocusConnector.Connect(SpeakerMixer, Speaker);
            FocusConnector.Connect(Microphone, MicropohoneMixer);
        }

        private PhoneCallAudioReceiver CallReceiver;
        private PhoneCallAudioSender CallSender;

        public MediaConnector Connector { get; private set; }

        public AudioMixerMediaHandler IncomingAudioMixer { get; private set; }
        public AudioMixerMediaHandler OutgoingAudioMixer { get; private set; }

        public IPhoneCall ItsCall { get; private set; }

        public CallSoundEngine(IPhoneCall call)
        {
            CallReceiver = new PhoneCallAudioReceiver();
            CallSender = new PhoneCallAudioSender();

            Connector = new MediaConnector();

            IncomingAudioMixer = new AudioMixerMediaHandler();
            OutgoingAudioMixer = new AudioMixerMediaHandler();

            Connector.Connect(CallReceiver, IncomingAudioMixer);
            Connector.Connect(OutgoingAudioMixer, CallSender);

            CallReceiver.AttachToCall(call);
            CallSender.AttachToCall(call);

            ItsCall = call;
        }

        public void Focus()
        {
            FocusConnector.Connect(IncomingAudioMixer, SpeakerMixer);
            FocusConnector.Connect(MicropohoneMixer, OutgoingAudioMixer);
        }

        public void Defocus()
        {
            FocusConnector.Disconnect(IncomingAudioMixer, SpeakerMixer);
            FocusConnector.Disconnect(MicropohoneMixer, OutgoingAudioMixer);
        }

        public void Dispose()
        {
            Defocus();
            IncomingAudioMixer.Dispose();
            OutgoingAudioMixer.Dispose();
        }
    }
}
