﻿using System.Collections.Concurrent;
using Ozeki.Media.MediaHandlers;
using Ozeki.VoIP;
using Voice_IVR.Model.InputInterpreter;
using System.Collections.Generic;

namespace Voice_IVR.Model.IVRMenuElements
{
    internal class WavePlayerElement : BaseIVRMenuElement
    {
        public string FileName;
        private WaveStreamPlayback WaveStreamPlayback;

        public WavePlayerElement(Dictionary<IPhoneCall, CallSoundEngine> soundEngines, 
            string fileName, IInputInterpreter inputInterpreter = null)
            : base(soundEngines, inputInterpreter)
        {
            FileName = fileName;
            WaveStreamPlayback = new WaveStreamPlayback(FileName);
        }

        /// <summary>
        /// DeActivate of the engine
        /// </summary>
        public override void EngineStop()
        {
            PhoneCall.CallStateChanged -= PhoneCallCallStateChanged;
            if (InputInterpreter != null) InputInterpreter.InputInformation -= InputInformation;
            
            DeActivateInputInterpreter();
            IVREngine.OnEngineEvent("WavePlayer", PhoneCall.DialInfo.UserName, " Stop");
        }

        /// <summary>
        /// Activation of the engine
        /// </summary>
        /// <param name="phoneCall"></param>
        public override void EngineExecute(IPhoneCall phoneCall)
        {
            PhoneCall = phoneCall;

            if (phoneCall != null && phoneCall.CallState == CallState.InCall)
            {
                IVREngine.OnEngineEvent("WavePlayer", phoneCall.DialInfo.UserName, " Start");
                phoneCall.CallStateChanged += PhoneCallCallStateChanged;

                CallSoundEngine engine = GetSoundEngine(phoneCall);
                if (engine != null)
                {
                    engine.Connector.Connect(WaveStreamPlayback, engine.OutgoingAudioMixer);
                    WaveStreamPlayback.StopStreaming();
                    WaveStreamPlayback.StartStreaming();
                }

                ActivateInputInterpreter();
            }
        }

        /// <summary>
        /// Handles the CallStateChanged event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhoneCallCallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            if (e.Item != CallState.InCall)
            {
                IPhoneCall call = (IPhoneCall)sender;

                WaveStreamPlayback.StopStreaming();
                CallSoundEngine engine = GetSoundEngine(PhoneCall);
                if (engine != null)
                    engine.Connector.Disconnect(WaveStreamPlayback, engine.OutgoingAudioMixer);

                EngineStop();
                IVREngine.OnEngineEvent("WavePlayer", PhoneCall.DialInfo.UserName, "Hangup");
            }
        }

        #region Cloning

        public override object Clone()
        {
            BaseIVRMenuElement playerElement = new WavePlayerElement(ItsSoundEngines, FileName);

            BaseCloning(playerElement);

            return playerElement;
        }

        #endregion
    }
}