﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ozeki.Media;
using System.Collections.Concurrent;
using Ozeki.VoIP;
using Ozeki.Media.MediaHandlers;
using Ozeki.VoIP.Media;

namespace Voice_IVR.Model.InputInterpreter
{
    sealed class SpeechToTextInputInterpreter : IInputInterpreter
    {
        private SpeechToText SpeechToText;
        private CallSoundEngine Engine;
        private string[] Choices;

        private Dictionary<IPhoneCall, CallSoundEngine> ItsSoundEngines;

        public SpeechToTextInputInterpreter(
            Dictionary<IPhoneCall, CallSoundEngine> soundEngines,
            string[] choices
            )
        {
            ItsSoundEngines = soundEngines;
            SpeechToText = SpeechToText.CreateInstance(new  AudioFormat(8000, 1, 16, 20), choices
            );
            if (SpeechToText == null)
                return;

            Choices = choices;
            SpeechToText.WordsHypotyzed += (SpeechToText_WordsHypotyzed);
            SpeechToText.WordsRecognized += (SpeechToText_WordsRecognized);
        }

        void SpeechToText_WordsRecognized(object sender, VoIPEventArgs<IEnumerable<string>> e)
        {
            if (InputInformation != null)
                InputInformation(this, new InputinformationArgs(e.Item.First()));
        }

        void SpeechToText_WordsHypotyzed(object sender, VoIPEventArgs<IEnumerable<string>> e)
        {
            if (InputInformation != null)
                InputInformation(this, new InputinformationArgs(e.Item.First()));
        }

        public event EventHandler<InputinformationArgs> InputInformation;

        public void Start(IPhoneCall phoneCall)
        {
            Engine = null;
            if (!ItsSoundEngines.TryGetValue(phoneCall, out Engine))
                return;

            Engine.Connector.Connect(Engine.IncomingAudioMixer, SpeechToText);
        }

        public void Stop()
        {
            Engine.Connector.Disconnect(Engine.IncomingAudioMixer, SpeechToText);
        }

        public object Clone()
        {
            return new SpeechToTextInputInterpreter(ItsSoundEngines, Choices);
        }
    }
}
